from lighthouse import Task, Header, Field, ValueType, EnterParamCollection, EnterParamField
from requests import Session
from datetime import datetime


class ResponsesHeader(metaclass=Header):
    display_name = 'Responses'

    RequestUrl = Field('URL', ValueType.String)
    Date = Field('Date', ValueType.Datetime)
    StatusCode = Field('Status code', ValueType.Integer)
    ReasonPhrase = Field('Reason phrase', ValueType.String)
    CodeOk = Field('Request success', ValueType.Boolean)

    ContentType = Field('Content-Type', ValueType.String)
    ContentLength = Field('Content-Length', ValueType.Integer)
    Server = Field('Server', ValueType.String)


class ExampleTask(Task):
    def get_id(self):
        return '6496a2cf-8efa-493c-a68d-b95c5a212c38'

    def get_display_name(self):
        return 'ResultWriter example'

    def get_category(self):
        return 'Tutorial tasks'

    def get_description(self):
        return 'This tutorial task explains how to write execution results'

    def get_headers(self):
        ResponsesHeader.set_property(ResponsesHeader.Date.system_name, 'datetime_format', 'yyyy/MM/dd HH:mm:ss')
        return ResponsesHeader

    def get_enter_params(self):
        return EnterParamCollection(
            EnterParamField('urls', 'URLs', ValueType.String, required=True, is_array=True)
        )

    def execute(self, enter_params, result_writer, log_writer, temp_directory):
        session = Session()
        for url in enter_params.urls:  # 'urls' marked as array, so this parameter will always be a list of values
            log_writer.info('Requesting resource: {}', url)  # messages can be formatted using *args
            try:
                response = session.get(url)  # perform HTTP GET request and receive server response
            except Exception as e:
                log_writer.info('Error performing request:')
                log_writer.info(e)  # .error() is for fatal errors and will abort further execution
                continue

            line = ResponsesHeader.create_empty()  # create a row for table

            line[ResponsesHeader.RequestUrl] = url
            # if field has ValueType.Datetime type, you can pass python datetime object
            line[ResponsesHeader.Date] = datetime.now()
            line[ResponsesHeader.StatusCode] = response.status_code
            line[ResponsesHeader.ReasonPhrase] = response.reason
            line[ResponsesHeader.CodeOk] = response.ok
            line[ResponsesHeader.ContentType] = response.headers.get('Content-Type')
            line[ResponsesHeader.ContentLength] = response.headers.get('Content-Length')
            line[ResponsesHeader.Server] = response.headers.get('Server')

            # this is where data is put to the table
            result_writer.write_line(line, header_class=ResponsesHeader)


if __name__ == '__main__':
    import sys

    class EnterParameters:
        urls = ['https://lampyre.io']

    class ResultWriterSub:
        @staticmethod
        def write_line(line: dict, header_class: Header):
            print(','.join([str(v) for v in line.values()]))

    class LogWriterSub:
        @staticmethod
        def info(message: str, *args):
            print(message.format(args))

        @staticmethod
        def error(message: str, *args):
            print(message.format(args), file=sys.stderr)

    temp_dir = '.'

    task = ExampleTask()

    task.execute(EnterParameters, ResultWriterSub, LogWriterSub, temp_dir)
